import unittest
import json

from audiobook import AudioBook


def load_json(filename):
    with open(filename, "r") as fp:
        return json.load(fp)


output = load_json("assets/output.json")
output_txt = (output['extracted_txt'], {'book_name': 'sample', 'pages': 1})

ab = AudioBook(speed="normal")


class TestAudioBook(unittest.TestCase):

    def test_txt_to_json_pdf_miner(self):
        self.assertEqual(ab.create_json_book("assets/sample.txt"), output_txt)

    def test_pdf_to_json_pdf_miner(self):
        self.assertEqual(ab.create_json_book("assets/sample.pdf", extraction_engine="pdfminer"), output_txt)

    def test_pdf_to_json_pypdf2(self):
        self.assertEqual(ab.create_json_book("assets/sample.pdf", extraction_engine="pypdf2"), output_txt)

    def test_odt_to_json(self):
        self.assertEqual(ab.create_json_book("assets/sample.odt"), output_txt)

    def test_mobi_to_json(self):
        self.assertEqual(ab.create_json_book("assets/sample.mobi"), output_txt)

    # def test_docs_to_json(self):
    #     self.assertEqual(ab.create_json_book("assets/sample.doc"), (output['docs'], {'book_name': 'sample', 'pages': 1}))

    # def test_epub_to_json(self): # epub test failing
    #     self.assertEqual(ab.create_json_book("assets/sample.epub"), (output['epub'], {'book_name': 'sample', 'pages': 1}))



import logging
import os

import pyttsx3
from tqdm import tqdm

from audiobook.config import speed_dict
from audiobook.utils import (
    docs_to_json,
    epub_to_json,
    html_to_json,
    load_json,
    mobi_to_json,
    odt_to_json,
    pdf_to_json,
    speak_text,
    txt_to_json,
    write_json_file,
)

logger = logging.getLogger("PyPDF2")
logger.setLevel(logging.INFO)

expand_usr = os.path.expanduser("~")
BOOK_DIR = os.path.join(expand_usr, "audiobook/library")
os.makedirs(BOOK_DIR, exist_ok=True)


class AudioBook(object):
    """
    AudioBook class

    methods:
        get_library: get all books in library
        create_json_book: Creates json book from input file by calling respective method  # noqa: E501
        save_audio: method to save audio files in folder
        read_book: reads the book

    sample usage:
        ab = AudioBook(speed="normal")
        ab.read_book(file_path, password="abcd")
    """

    def __init__(self, speed="normal", volume=1.0):
        self.engine = pyttsx3.init()
        self.engine.setProperty("rate", speed_dict[speed])
        self.engine.setProperty("volume", volume)

    def get_library(self):
        """get all books in library"""
        total_books = os.listdir(BOOK_DIR)
        if len(total_books) == 0:
            return "You have no books in your library"
        print(
            "You Have total {} books in your library".format(len(total_books))
        )
        return total_books

    def create_json_book(self, input_book_path, password=None, extraction_engine=None, load_from_library=False):
        """method to create json book from input file
        it calls respective method based on file format"""
        json_filename = (
            os.path.basename(input_book_path).split(".")[0] + ".json"
        )

        if load_from_library:
            print("Loading book from library")
            if os.path.exists(os.path.join(BOOK_DIR, json_filename)):
                metadata = {"book_name": json_filename.split(".")[0]}
                print("Book already exists in library, reading from library")
                json_book = load_json(os.path.join(BOOK_DIR, json_filename))
                metadata["pages"] = len(json_book)
                return json_book, metadata

        elif input_book_path.endswith(".odt"):
            json_book, metadata = odt_to_json(input_book_path)
        elif input_book_path.endswith(".pdf"):
            json_book, metadata = pdf_to_json(input_book_path, password, extraction_engine=extraction_engine)
        elif input_book_path.endswith(".txt"):
            json_book, metadata = txt_to_json(input_book_path)
        elif input_book_path.endswith(".epub"):
            json_book, metadata = epub_to_json(input_book_path)
        elif input_book_path.endswith(".mobi"):
            json_book, metadata = mobi_to_json(input_book_path)
        elif input_book_path.startswith("http"):
            json_book, metadata = html_to_json(input_book_path)
        elif input_book_path.endswith((".docx", ".doc")):
            json_book, metadata = docs_to_json(input_book_path)
        else:
            raise NotImplementedError("Only PDF, TXT, EPUB, MOBI, ODT, HTTP, DOCX and DOC files are supported")

        write_json_file(json_book, os.path.join(BOOK_DIR, json_filename))

        return json_book, metadata

    def save_audio(self, input_book_path, password=None, save_page_wise=False, extraction_engine=None):
        """method to save audio files in folder"""

        json_book, metadata = self.create_json_book(input_book_path, password, extraction_engine)

        book_name = metadata["book_name"]
        book_dir = os.path.join(BOOK_DIR, book_name)
        os.makedirs(book_dir, exist_ok=True)

        print("Saving audio files in folder: {}".format(book_dir))

        if save_page_wise:
            for page_num, text in tqdm(json_book.items()):
                self.engine.save_to_file(
                    text,
                    os.path.join(
                        book_name,
                        book_name + "_page_" + (str(page_num)) + ".mp3",
                    ),
                )
                self.engine.runAndWait()

        elif not save_page_wise:
            all_text = " ".join([text for text in json_book.values()])
            self.engine.save_to_file(
                all_text, os.path.join(book_name, book_name + ".mp3")
            )
            self.engine.runAndWait()

    def read_book(self, input_book_path, password=None, extraction_engine=None):
        """method to read the book

        input_book_path: filepath, url path or book name
        """
        json_book, metadata = self.create_json_book(input_book_path, password, extraction_engine)

        pages = metadata["pages"]

        speak_text(self.engine, f"The book has total {str(pages)} pages!")
        speak_text(
            self.engine, "Please enter the page number: ", display=False
        )
        start_page = int(input("Please enter the page number: ")) - 1

        reading = True
        while reading:
            if start_page > pages or start_page < 0:
                speak_text(self.engine, "Invalid page number!")
                speak_text(
                    self.engine, f"The book has total {str(pages)} pages!"
                )
                start_page = int(input("Please enter the page number: "))

            speak_text(self.engine, f"Reading page {str(start_page+1)}")
            pageText = json_book[str(start_page)]
            speak_text(self.engine, pageText, display=False)

            input_message = "Please Select an option: \n "
            "1. Type 'r' to read again: \n "
            "2. Type 'p' to read previous page\n "
            "3. Type 'n' to read next page\n "
            "4. Type 'q' to quit:\n "
            "5. Type page number to read that page:\n"

            user_input = input(input_message)
            if user_input == "r":
                speak_text(self.engine, f"Reading page {str(start_page+1)}")
                continue
            elif user_input == "p":
                speak_text(self.engine, "Reading previous page")
                start_page -= 1
                continue
            elif user_input == "n":
                speak_text(self.engine, "Reading next page")
                start_page += 1
                continue
            elif user_input == "q":
                speak_text(self.engine, "Quitting the book!")
                break
            elif user_input.isnumeric():
                start_page = int(user_input) - 1
            else:
                user_input = input(input_message)
                continue



import argparse
import os.path

from audiobook.main import AudioBook, BOOK_DIR

__version__ = "3.0.0"


def main():
    parser = argparse.ArgumentParser(description="AudioBook - listen to any PDF book")
    parser.add_argument("-p", "--path", nargs="?", default=None, help="book file path")
    parser.add_argument('-v', '--version', action='version', version=__version__)

    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-l", "--library", action="store_true", help="get all books in library"
    )
    group.add_argument(
        "-c",
        "--create-json",
        action="store_true",
        help="create json file from input file",
    )
    group.add_argument(
        "-s",
        "--save-audio",
        action="store_true",
        help="save audio files from input file",
    )
    group.add_argument(
        "-r", "--read-book", action="store_true", help="read the book from input file")

    args = parser.parse_args()

    if args.path and not os.path.isfile(args.path):
        if os.path.isfile(os.path.join(BOOK_DIR, args.path)):
            args.path = os.path.join(BOOK_DIR, args.path)
        else:
            print(f"{args.path} is not a valid file")

    ab = AudioBook()
    if args.library:
        ab.get_library()
    elif args.create_json and args.path:
        ab.create_json_book(args.path)
    elif args.save_audio and args.path:
        ab.save_audio(args.path)
    elif args.path:
        ab.read_book(args.path)
    else:
        ab.get_library()
        print("Use `python audiobook -h` to see all valid options")


if __name__ == "__main__":
    main()



import json
import os
import re

import docx2txt
import ebooklib
import html2text
import mobi
from bs4 import BeautifulSoup
from ebooklib import epub
from odf import text, teletype
from odf.opendocument import load

from audiobook.doc_parser.web_parser import ArticleWebScraper
from audiobook.doc_parser.pdf_parser import PyPDF2DocParser
from audiobook.doc_parser.pdf_parser import PdfMinerDocParser


def load_json(filename):
    with open(filename, "r") as fp:
        return json.load(fp)


def write_json_file(json_data, filename):
    with open(filename, "w") as fp:
        json.dump(json_data, fp)


def text_preprocessing(input_text):
    """function to preprocess text"""
    regex = re.compile(r"[\n\r\t]")
    preprocessed_text = regex.sub("", input_text)
    preprocessed_text = re.sub(" +", " ", preprocessed_text)
    preprocessed_text = preprocessed_text.strip()
    return preprocessed_text


def response_to_text(chapter):
    """fuction to convert response to text

    required for epub files
    maybe required for html files
    """
    soup = BeautifulSoup(chapter, "html.parser")
    extracted_text = [para.get_text() for para in soup.find_all("p")]
    extracted_text = " ".join(extracted_text)
    preprocessed_text = text_preprocessing(extracted_text)
    return preprocessed_text


def speak_text(engine, text, display=True):
    """function to speak text and display it"""
    if display:
        print(text)
    engine.say(text)
    engine.runAndWait()


def mobi_to_json(input_book_path):
    """sub method to create json book from mobi file"""
    metadata = {}
    json_book = {}
    book_name = os.path.basename(input_book_path).split(".")[0]
    tempdir, filepath = mobi.extract(input_book_path)
    with open(filepath, "r", encoding="utf-8") as fp:
        content = fp.read()
    book_data = html2text.html2text(content)
    book_data = text_preprocessing(book_data)

    for i in range(0, len(book_data), 2000):
        page_num = i // 2000
        json_book[str(page_num)] = book_data[i: i + 2000]

    metadata["pages"] = len(json_book)
    metadata["book_name"] = book_name
    return json_book, metadata


def pdf_to_json(input_book_path, password=None, extraction_engine="pypdf2"):
    """sub method to create json book from pdf file"""
    metadata = {}
    json_book = {}
    basename = os.path.basename(input_book_path).split(".")[0]
    if extraction_engine is None or extraction_engine == "pdfminer":
        print("Using pdfminer")
        pdf_parser = PdfMinerDocParser()
    elif extraction_engine == "pypdf2":
        print("Using pypdf2")
        pdf_parser = PyPDF2DocParser()
    else:
        raise NotImplementedError("Only pdfminer and pypdf2 are supported")

    text = pdf_parser.get_text(input_book_path, password=password)
    text = text_preprocessing(text)

    for i in range(0, len(text), 2000):
        page_num = i // 2000
        json_book[str(page_num)] = text[i: i + 2000]

    metadata['book_name'] = basename
    metadata['pages'] = len(json_book)
    return json_book, metadata


def odt_to_json(input_book_path):
    """sub method to create json book from odt file"""
    metadata = {}
    json_book = {}
    basename = os.path.basename(input_book_path).split(".")[0]

    textdoc = load(input_book_path)
    allparas = textdoc.getElementsByType(text.P)
    output_text = ""
    for i in range(len(allparas)):
        output_text += " " + teletype.extractText(allparas[i])
    output_text = text_preprocessing(output_text)

    for i in range(0, len(output_text), 2000):
        page_num = i // 2000
        json_book[str(page_num)] = output_text[i: i + 2000]

    metadata['book_name'] = basename
    metadata['pages'] = len(json_book)

    return json_book, metadata


def txt_to_json(input_book_path):
    """sub method to create json book from txt file"""
    json_book = {}
    metadata = {}
    book_name = os.path.basename(input_book_path).split(".")[0]
    with open(input_book_path, "r") as fp:
        file_txt_data = fp.read()
    file_txt_data = text_preprocessing(file_txt_data)

    for i in range(0, len(file_txt_data), 2000):
        page_num = i // 2000
        json_book[str(page_num)] = file_txt_data[i: i + 2000]

    metadata["pages"] = len(json_book)
    metadata["book_name"] = book_name
    return json_book, metadata


def docs_to_json(input_book_path):
    """sub method to create json book from docs file"""
    metadata = {}
    json_book = {}
    book_name = os.path.basename(input_book_path).split(".")[0]
    book_data = docx2txt.process(input_book_path)
    for i in range(0, len(book_data), 2000):
        page_num = i // 2000
        json_book[str(page_num)] = book_data[i: i + 2000]

    metadata["pages"] = len(json_book)
    metadata["book_name"] = book_name
    return json_book, metadata


def epub_to_json(input_book_path):
    metadata = {}
    json_book = {}
    book_name = os.path.basename(input_book_path).split(".")[0]
    book = epub.read_epub(input_book_path)
    text = " ".join(
        [
            response_to_text(chapter.get_body_content())
            for chapter in book.get_items_of_type(ebooklib.ITEM_DOCUMENT)
        ]
    )
    for i in range(1, len(text) + 1, 2000):
        page_num = i // 2000
        json_book[str(page_num)] = text[i: i + 2000]

    metadata["pages"] = len(json_book)
    metadata["book_name"] = book_name
    return json_book, metadata


def html_to_json(url):
    """method to create json book from web article"""
    metadata = {}
    json_book = {}
    book_name = os.path.basename(url).split(".")[0]
    article_scraper = ArticleWebScraper(url)
    page_data = article_scraper.get_page_data()
    page_data = text_preprocessing(page_data)
    for i in range(0, len(page_data), 2000):
        page_num = i // 2000
        json_book[str(page_num)] = page_data[i: i + 2000]

    metadata["pages"] = len(json_book)
    metadata["book_name"] = book_name
    return json_book, metadata



import io
import ast

import PyPDF2

from pdfminer.pdfinterp import PDFResourceManager
from pdfminer.pdfinterp import PDFPageInterpreter
from pdfminer.converter import TextConverter
from pdfminer.layout import LAParams
from pdfminer.pdfpage import PDFPage

from pdfminer.pdfparser import PDFParser
from pdfminer.pdfdocument import PDFDocument


class PdfMinerDocParser(object):
    """

    PdfMinor Doc Parser:
        1. get_metadata : get metadata of pdf file
        2. get_text : convert pdf to text
        3. get_toc : get table of contents if available

    """
    def __init__(self):
        self.laparams = LAParams(char_margin=1,
                                 line_margin=0.5,
                                 all_texts=True)

    def get_metadata(self):
        pass

    def get_text(self, filepath, password=None, maxpages=0, caching=True):
        """ function to read all the text from pdf file """
        pagenos = set()
        rsrcmgr = PDFResourceManager()
        retstr = io.StringIO()

        device = TextConverter(rsrcmgr, retstr, laparams=self.laparams)

        with open(filepath, 'rb') as fp:
            interpreter = PDFPageInterpreter(rsrcmgr, device)

            for page in PDFPage.get_pages(fp=fp,
                                          pagenos=pagenos,
                                          maxpages=maxpages,
                                          password=password,
                                          caching=caching):
                interpreter.process_page(page)

            pdf_data = retstr.getvalue()
            return pdf_data

    def get_toc(self, filepath, password=None):
        """ function to get table of contents if available """
        output_toc = []
        # if not os.path.isfile(output_toc_path):
        with open(filepath, 'rb') as fp:
            parser = PDFParser(fp)

            document = PDFDocument(parser, password=password)
            try:
                outlines = document.get_outlines()
                for (level, title, _, _, _) in outlines:
                    output_toc.append((level, title))
                return output_toc
            except Exception as e:
                print(e)
                return output_toc


class PyPDF2DocParser(object):
    """
    PyPdf2 Doc Parser:

    methods:
        1. get_metadata : get metadata of pdf file
        2. get_text : convert pdf to text
        3. get_toc : get table of contents if available

    """
    def __init__(self):
        pass

    def get_metadata(self):
        pass

    def get_text(self, filepath, password=None, maxpages=0):
        """ function to read all the text from pdf file """
        pdf_data = ""
        with open(filepath, "rb") as fp:
            pdfReader = PyPDF2.PdfFileReader(fp)
            if password:
                pdfReader.decrypt(password)
            num_pages = pdfReader.numPages
            if maxpages:
                num_pages = min(num_pages, maxpages)
            for i in range(num_pages):
                pageObj = pdfReader.getPage(i)
                pdf_data += pageObj.extractText()
        return pdf_data

    def get_toc(self, filepath, password=None):
        outlines = []

        with open(filepath, "rb") as fp:
            pdfReader = PyPDF2.PdfFileReader(fp, strict=False)
            if password:
                pdfReader.decrypt(password)
            outlines = pdfReader.getOutlines()
            if outlines:
                outlines = str(outlines).replace("IndirectObject(", "[")
                outlines = outlines.replace(")", "]").replace("/", "")
                outlines = ast.literal_eval(outlines)

        return outlines



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    required = f.read().splitlines()

setuptools.setup(
    name="audiobook",
    version="3.0.0",
    author="CodePerfectPlus",
    author_email="deepak008@live.com",
    description="Listen to your favourite audiobook",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=required,
    url="https://github.com/codePerfectPlus/audiobook",
    keywords="audiobook",
    packages=setuptools.find_packages(),
    project_urls= {
        "Documentation": "https://pycontributors.readthedocs.io/projects/Audiobook/en/latest/",
        "Source": "https://github.com/Py-Contributors/AudioBook",
        "Tracker": "https://github.com/Py-Contributors/AudioBook/issues"
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
    ],
    python_requires=">=3.4",
    entry_points={
        "console_scripts": ["audiobook = audiobook.cli:main"],
    },
)



import requests
from bs4 import BeautifulSoup


class ArticleWebScraper:
    """
    ArticleWebScraper class

    methods:
        get_json_from_web_article: returns a json from a non-empty <article> tag  # noqa: E501
        get_title_from_article: returns the <title> tag from the html page

    sample usage:
        ab = AudioBook(speed="normal")
        ab.read_book(file_path, password="abcd")
    """

    def __init__(self, article_url):
        page = requests.get(article_url)
        self.article_url = article_url
        self.soup = BeautifulSoup(page.content, "html.parser")

    def get_title_from_article(self):
        """returns the <title> tag from the html page"""
        return self.soup.title.text

    def get_page_data(self):
        """returns a json from a non-empty <article> tag"""
        response = requests.get(self.article_url)

        if response.status_code != 200:
            return None

        soup = BeautifulSoup(response.content, "html.parser")

        text_data = soup.getText().replace("\n", "")
        return text_data
